#!/usr/bin/perl -wT

# Copyright (C) 1994 - 2001  eXtropia.com
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.

use strict;

use lib qw(
    ./Modules
    ./ActionHandlers
    ./ActionHandlers/MLM
    ./Views/Extropia/WebDB
    ./Views/Extropia/WebDB/MLM
    ./Views/Extropia/AuthManager
    ./Views/Extropia/StandardTemplates
);

          # The following is only of interest to mod_perl, PerlEx,
          # and other Perl acceleration users

unshift @INC, qw(
    ./Modules
    ./ActionHandlers
    ./ActionHandlers/MLM
    ./Views/Extropia/WebDB
    ./Views/Extropia/WebDB/MLM
    ./Views/Extropia/AuthManager
    ./Views/Extropia/StandardTemplates
    ./Modules

use CGI qw(-debug);
use CGI::Carp qw(fatalsToBrowser);

use Extropia::App::DBApp;
use Extropia::View;
use Extropia::SessionManager;

my $CGI = new CGI() or
    die("Unable to construct the CGI object" .
        ". Please contact the webmaster.");

my $VIEW_LOADER = new Extropia::View() or
    die("Unable to construct the VIEW LOADER object in " . $CGI->script_name() .
        ". Please contact the webmaster.");

my $DATAFILES_DIRECTORY = "./Datafiles";

foreach ($CGI->param()) {
    $CGI->param($1,$CGI->param($_)) if (/(.*)\.x/);
}
                
######################################################################
#                          SESSION SETUP                             #
######################################################################

my @SESSION_CONFIG_PARAMS = (
    -TYPE            => 'File',
    -MAX_MODIFY_TIME => 60 * 60,
    -SESSION_DIR     => "$DATAFILES_DIRECTORY/Sessions",
    -FATAL_TIMEOUT   => 0,
    -FATAL_SESSION_NOT_FOUND => 0
);

######################################################################
#                     SESSION MANAGER SETUP                          #
######################################################################

my @SESSION_MANAGER_CONFIG_PARAMS = (
    -TYPE           => 'FormVar',
    -CGI_OBJECT     => $CGI,
    -SESSION_PARAMS => \@SESSION_CONFIG_PARAMS
);

my $SESSION_MGR = Extropia::SessionManager->create(
    @SESSION_MANAGER_CONFIG_PARAMS
);

my $SESSION     = $SESSION_MGR->createSession();

######################################################################
#                       AUTHENTICATION SETUP                         #
######################################################################

my @AUTH_USER_DATASOURCE_FIELD_NAMES = qw(
    username
    password
    groups
    firstname
    lastname
    email
);

my @AUTH_USER_DATASOURCE_PARAMS = (
    -TYPE                       => 'File',
    -FIELD_DELIMITER            => '|',
    -FIELD_NAMES                => \@AUTH_USER_DATASOURCE_FIELD_NAMES,
    -CREATE_FILE_IF_NONE_EXISTS => 1,
    -FILE                       => "$DATAFILES_DIRECTORY/mlm.users.dat"
);

my @AUTH_ENCRYPT_PARAMS = (
    -TYPE => 'Crypt'
);

my %USER_FIELDS_TO_DATASOURCE_MAPPING = (
    'auth_username'  => 'username',
    'auth_password'  => 'password',
    'auth_firstname' => 'firstname',
    'auth_lastname'  => 'lastname',
    'auth_groups'    => 'groups',
    'auth_email'     => 'email'
);

my @AUTH_CACHE_PARAMS = (
    -TYPE           => 'Session',
    -SESSION_OBJECT => $SESSION
);

my @AUTH_CONFIG_PARAMS = (
    -TYPE                                => 'DataSource',
    -USER_DATASOURCE_PARAMS              => \@AUTH_USER_DATASOURCE_PARAMS,
    -ENCRYPT_PARAMS                      => \@AUTH_ENCRYPT_PARAMS,
    -ADD_REGISTRATION_TO_USER_DATASOURCE => 1,
    -USER_FIELDS_TO_DATASOURCE_MAPPING   => \%USER_FIELDS_TO_DATASOURCE_MAPPING,
    -AUTH_CACHE_PARAMS                   => \@AUTH_CACHE_PARAMS
);

######################################################################
#                 AUTHENTICATION MANAGER SETUP                       #
######################################################################

my @AUTH_VIEW_DISPLAY_PARAMS = (
    -COLOR_FOR_EVEN_ROWS     => 'E5E5E5',
    -COLOR_FOR_ODD_ROWS      => 'FFFFFF',
    -CSS_VIEW_URL            => $CGI->script_name() . "?display_css_view=on",
    -APPLICATION_LOGO        => 'logo.gif',
    -APPLICATION_LOGO_HEIGHT => '40',
    -APPLICATION_LOGO_WIDTH  => '353',
    -APPLICATION_LOGO_ALT    => 'WebDB Demo',
    -HTTP_HEADER_PARAMS      => [-EXPIRES => '-1d'],
    -DOCUMENT_ROOT_URL       => '/',
    -IMAGE_ROOT_URL          => '/Images/Extropia',
    -LINK_TARGET             => '_self',
    -SCRIPT_DISPLAY_NAME     => 'MLM',
    -SCRIPT_NAME             => $CGI->script_name(),
    -PAGE_TOP_VIEW           => 'PageTopView',
    -PAGE_BOTTOM_VIEW        => 'PageBottomView'
);

my @AUTH_REGISTRATION_DH_MANAGER_PARAMS = (
    -TYPE         => 'CGI',
    -CGI_OBJECT   => $CGI,
    -DATAHANDLERS => [qw(
        Email
        Exists
    )],
    
    -FIELD_MAPPINGS => {
                'auth_username'     => 'Username',
                'auth_password'     => 'Password',
                'auth_password2'    => 'Confirm Password',
                'auth_firstname'    => 'First Name',
                'auth_lastname'     => 'Last Name',
                'auth_email'        => 'E-Mail Address'
        },
  
        -IS_FILLED_IN => [qw(
                auth_username
                auth_firstname
                auth_lastname
                auth_email
        )],
    
        -IS_EMAIL => [qw(
                auth_email
        )]
);
    
my @USER_FIELDS = (qw(
    auth_username
    auth_password
    auth_groups
    auth_firstname
    auth_lastname
    auth_email
));

my %USER_FIELD_NAME_MAPPINGS = (
    'auth_username'  => 'Username',
    'auth_password'  => 'Password',
    'auth_groups'    => 'Groups',
    'auth_firstname' => 'First Name',
    'auth_lastname'  => 'Last Name',
    'auth_email'     => 'E-Mail'
);

my %USER_FIELD_TYPES = (
    -USERNAME_FIELD => 'auth_username',
    -PASSWORD_FIELD => 'auth_password',
    -GROUP_FIELD    => 'auth_groups',
    -EMAIL_FIELD    => 'auth_email'
);

my @MAIL_PARAMS = (
    -TYPE         => 'Sendmail',
);

my @USER_MAIL_SEND_PARAMS = (
    -FROM    => 'you@yourdomain.com',
    -SUBJECT => 'Password Generated'
);

my @ADMIN_MAIL_SEND_PARAMS = (
    -FROM    => 'you@yourdomain.com',
    -TO      => 'you@yourdomain.com',
    -SUBJECT => 'Registration Notification'
);                

my @AUTH_MANAGER_CONFIG_PARAMS = (
    -TYPE                        => 'CGI',
    -ADMIN_MAIL_SEND_PARAMS      => \@ADMIN_MAIL_SEND_PARAMS,
    -AUTH_VIEW_PARAMS            => \@AUTH_VIEW_DISPLAY_PARAMS,
    -MAIL_PARAMS                 => \@MAIL_PARAMS,
    -USER_MAIL_SEND_PARAMS       => \@USER_MAIL_SEND_PARAMS,
    -SESSION_OBJECT              => $SESSION,
    -AUTH_VIEWS                  => 'CGIViews.pm',
    -VIEW_LOADER                 => $VIEW_LOADER,
    -AUTH_PARAMS                 => \@AUTH_CONFIG_PARAMS,
    -CGI_OBJECT                  => $CGI,
    -ALLOW_REGISTRATION          => 1,   
    -ALLOW_USER_SEARCH           => 1,
    -USER_SEARCH_FIELD           => 'auth_email',
    -GENERATE_PASSWORD           => 0,
    -DEFAULT_GROUPS              => 'normal',
    -EMAIL_REGISTRATION_TO_ADMIN => 0,
    -USER_FIELDS                 => \@USER_FIELDS,
    -USER_FIELD_TYPES            => \%USER_FIELD_TYPES,
    -USER_FIELD_NAME_MAPPINGS    => \%USER_FIELD_NAME_MAPPINGS,
    -DISPLAY_REGISTRATION_AGAIN_AFTER_FAILURE => 1,
    -AUTH_REGISTRATION_DH_MANAGER_PARAMS => \@AUTH_REGISTRATION_DH_MANAGER_PARAMS
);

######################################################################
#                      DATA HANDLER SETUP                            #
######################################################################
                       
my @ADD_FORM_DHM_CONFIG_PARAMS = (
    -TYPE         => 'CGI',
    -CGI_OBJECT   => $CGI,  
    -DATAHANDLERS => [qw(
        Exists
        HTML
        Email
        Number
        String
        )],

    -FIELD_MAPPINGS => {
        'subject'   => 'Subject',
        'abstract'  => 'Abstract',
        'full_text' => 'Full Text'
    },

    -RULES => [
        -ESCAPE_HTML_TAGS => [
            -FIELDS => [qw(
                *
            )],
        ],

        -DOES_NOT_CONTAIN => [
            -FIELDS => [qw(
                *
            )],

            -CONTENT_TO_DISALLOW => '\\',
            -ERROR_MESSAGE => "You may not have a '\\' character in the " .
                              "%FIELD_NAME% field."
        ],

        -DOES_NOT_CONTAIN => [
            -FIELDS => [qw(
                *
            )],

            -CONTENT_TO_DISALLOW => '\"',
            -ERROR_MESSAGE => "You may not have a '\"' character in the " .
                              "%FIELD_NAME% field."
        ],

        -SUBSTITUTE_ONE_STRING_FOR_ANOTHER => [
            -FIELDS => [qw(
                *
            )],
            -ORIGINAL_STRING => '"',
            -NEW_STRING => "''"
        ],

        -IS_FILLED_IN => [
            -FIELDS => [qw(
                fname
                lname
                email
            )]
        ],

        -IS_EMAIL => [
            -FIELDS => [qw(
                email
            )]
        ],

        -IS_NUMBER=> [
            -FIELDS => [qw(
                age
            )]
        ]
    ]
);

my @MODIFY_FORM_DHM_CONFIG_PARAMS = (
    -TYPE         => 'CGI',
    -CGI_OBJECT   => $CGI,  
    -DATAHANDLERS => [qw(
        Exists
        Email
        Number
        HTML
        String
        )],

    -FIELD_MAPPINGS => {
        'subject'   => 'Subject',
        'abstract'  => 'Abstract',
        'full_text' => 'Full Text'
    },

    -RULES => [
        -ESCAPE_HTML_TAGS => [
            -FIELDS => [qw(
                *
            )],
        ],

        -DOES_NOT_CONTAIN => [
            -FIELDS => [qw(
                *
            )],

            -CONTENT_TO_DISALLOW => '\\',
            -ERROR_MESSAGE => "You may not have a '\\' character in the " .
                              "%FIELD_NAME% field."
        ],

        -DOES_NOT_CONTAIN => [
            -FIELDS => [qw(
                *
            )],

            -CONTENT_TO_DISALLOW => '\"',
            -ERROR_MESSAGE => "You may not have a '\"' character in the " .
                              "%FIELD_NAME% field."
        ],

        -SUBSTITUTE_ONE_STRING_FOR_ANOTHER => [
            -FIELDS => [qw(
                *
            )],
            -ORIGINAL_STRING => '"',
            -NEW_STRING => "''"
        ],

        -IS_FILLED_IN => [
            -FIELDS => [qw(
                fname
                lname
                email
            )]
        ],

        -IS_EMAIL => [
            -FIELDS => [qw(
                email
            )]
        ],

        -IS_NUMBER=> [
            -FIELDS => [qw(
                age
            )]
        ]
    ]
);

my @SEND_MAIL_FORM_DHM_CONFIG_PARAMS = (
    -TYPE         => 'CGI',
    -CGI_OBJECT   => $CGI,  
    -DATAHANDLERS => [qw(
        Exists
        HTML
        )],

    -FIELD_MAPPINGS => {
        'subject'   => 'Subject',
        'body' => 'Body'
    },

    -RULES => [
        -IS_FILLED_IN => [
            -FIELDS => [qw(
                subject
                body
            )]
        ]
    ]
);

my @DATA_HANDLER_MANAGER_CONFIG_PARAMS = (
    -ADD_FORM_DHM_CONFIG_PARAMS    => \@ADD_FORM_DHM_CONFIG_PARAMS,
    -MODIFY_FORM_DHM_CONFIG_PARAMS => \@MODIFY_FORM_DHM_CONFIG_PARAMS,
    -SEND_MAIL_FORM_DHM_CONFIG_PARAMS => \@SEND_MAIL_FORM_DHM_CONFIG_PARAMS
);

######################################################################
#                      DATASOURCE SETUP                              #
######################################################################

my @DATASOURCE_FIELD_NAMES = qw(
        record_id
        fname
        lname
        email
        gender
        age
        occupation
        username_of_poster
        group_of_poster
        date_time_posted
);

my @FIELDS_REQUIRED_FOR_DELETION = qw(
        email
);

my %BASIC_INPUT_WIDGET_DEFINITIONS = (
    fname => [
        -DISPLAY_NAME => 'First Name',
        -TYPE         => 'textfield',
        -NAME         => 'fname',
        -SIZE         => 30,
        -MAXLENGTH    => 80
    ],

    lname => [
        -DISPLAY_NAME => 'Last Name',
        -TYPE         => 'textfield',
        -NAME         => 'lname',
        -SIZE         => 30,
        -MAXLENGTH    => 80
    ],

    email => [
        -DISPLAY_NAME => 'Email',
        -TYPE         => 'textfield',
        -NAME         => 'email',
        -SIZE         => 30,
        -MAXLENGTH    => 80
    ],

    age => [
        -DISPLAY_NAME => 'Age',
        -TYPE         => 'textfield',
        -NAME         => 'age',
        -SIZE         => 30,
        -MAXLENGTH    => 80
    ],

    gender => [
        -DISPLAY_NAME => 'Gender',
        -TYPE         => 'popup_menu',
        -NAME         => 'gender',
        -VALUES       => [qw(Female Male)]
    ],

    occupation => [
        -DISPLAY_NAME => 'Occupation',
        -TYPE         => 'popup_menu',
        -NAME         => 'occupation',
        -VALUES       => [(
            'Software Engineer',
            'HTML Designer',
            'Administration',
            'Nursing',
            'Drummer'
        )]
    ],

);

my @BASIC_INPUT_WIDGET_DISPLAY_ORDER = qw(
        fname
        lname
        email
        gender
        age
        occupation
);

my %SEND_MAIL_FORM_INPUT_WIDGET_DEFINITIONS = (
    subject => [
        -DISPLAY_NAME => 'Subject',
        -TYPE         => 'textfield',
        -NAME         => 'subject',
        -SIZE         => 30,
        -MAXLENGTH    => 80
    ],

    body => [
        -DISPLAY_NAME => 'Message Body',
        -TYPE         => 'textarea',
        -NAME         => 'body',
        -ROWS         => 5,
        -COLS         => 35,
        -WRAP         => 'VIRTUAL'
    ],
);

my @SEND_MAIL_FORM_INPUT_WIDGET_DISPLAY_ORDER = qw(
    subject
    body
);

my @INPUT_WIDGET_DEFINITIONS = (
    -BASIC_INPUT_WIDGET_DEFINITIONS => \%BASIC_INPUT_WIDGET_DEFINITIONS,
    -BASIC_INPUT_WIDGET_DISPLAY_ORDER => \@BASIC_INPUT_WIDGET_DISPLAY_ORDER,
    -SEND_MAIL_FORM_INPUT_WIDGET_DEFINITIONS => \%SEND_MAIL_FORM_INPUT_WIDGET_DEFINITIONS,
    -SEND_MAIL_FORM_INPUT_WIDGET_DISPLAY_ORDER => \@SEND_MAIL_FORM_INPUT_WIDGET_DISPLAY_ORDER
);

my @BASIC_DATASOURCE_CONFIG_PARAMS = (
    -TYPE                       => 'File', 
    -CREATE_FILE_IF_NONE_EXISTS => 1,
    -FILE                       => "$DATAFILES_DIRECTORY/mlm.dat",
    -FIELD_DELIMITER            => '|',
    -COMMENT_PREFIX             => '#',
    -FIELD_NAMES                => \@DATASOURCE_FIELD_NAMES,
    -KEY_FIELDS                 => ['record_id'],
    -FIELD_TYPES                => {
        record_id => 'Autoincrement'
        }
);

my @DATASOURCE_CONFIG_PARAMS = (
    -BASIC_DATASOURCE_CONFIG_PARAMS     => \@BASIC_DATASOURCE_CONFIG_PARAMS,
    -AUTH_USER_DATASOURCE_CONFIG_PARAMS => \@AUTH_USER_DATASOURCE_PARAMS
);


######################################################################
#                          MAILER SETUP                              #
######################################################################
           
my @MAIL_CONFIG_PARAMS = (     
    -TYPE         => 'Sendmail'
);

my @EMAIL_DISPLAY_FIELDS = qw(
        fname
        lname
        email
        gender
        age
        occupation
);

my @DELETE_EVENT_MAIL_SEND_PARAMS = (
    -FROM     => 'you@yourdomain.com',
    -TO       => 'you@yourdomain.com',
    -REPLY_TO => 'you@yourdomain.com',
    -SUBJECT  => 'MLM Delete'
);

my @ADD_EVENT_MAIL_SEND_PARAMS = (
    -FROM     => 'you@yourdomain.com',
    -TO       => 'you@yourdomain.com',
    -REPLY_TO => 'you@yourdomain.com',
    -SUBJECT  => 'MLM Addition'
);

my @MODIFY_EVENT_MAIL_SEND_PARAMS = (
    -FROM     => 'you@yourdomain.com',
    -TO       => 'you@yourdomain.com',
    -REPLY_TO => 'you@yourdomain.com',
    -SUBJECT  => 'MLM Modification'
);

my @SEND_EVENT_MAIL_SEND_PARAMS = (
    -FROM     => 'you@yourdomain.com',
    -REPLY_TO => 'you@yourdomain.com'
);

my @MAIL_SEND_PARAMS = (
    -DELETE_EVENT_MAIL_SEND_PARAMS => \@DELETE_EVENT_MAIL_SEND_PARAMS,
    -ADD_EVENT_MAIL_SEND_PARAMS    => \@ADD_EVENT_MAIL_SEND_PARAMS,
    -MODIFY_EVENT_MAIL_SEND_PARAMS => \@MODIFY_EVENT_MAIL_SEND_PARAMS,
    -SEND_EVENT_MAIL_SEND_PARAMS   => \@SEND_EVENT_MAIL_SEND_PARAMS
);

##################################################################
#                         LOGGING SETUP                          #
##################################################################

my @LOG_CONFIG_PARAMS = (
    -TYPE             => 'File',
    -LOG_FILE         => "$DATAFILES_DIRECTORY/mlm.log",
    -LOG_ENTRY_SUFFIX => '|' . _generateEnvVarsString() . '|',
    -LOG_ENTRY_PREFIX => 'MLM|'
);

sub _generateEnvVarsString {
    my @env_values;
    
    my $key;
    foreach $key (keys %ENV) {
        push (@env_values, "$key=" . $ENV{$key});
    }   
    return join ("\|", @env_values);
}   
   
######################################################################
#                          VIEW SETUP                                #
######################################################################

my @VALID_VIEWS = qw(
    CSSView
    AddAcknowledgementView
    AddRecordConfirmationView
    DeleteRecordConfirmationView
    DeleteAcknowledgementView
    ModifyAcknowledgementView
    ModifyRecordConfirmationView
    SessionTimeoutErrorView
    AddRecordView
    PowerSearchFormView
    DetailsView
    ModifyRecordView
    MLMTOCView
    MLMFindRecordFormView
    MLMSendMailFormView
    MLMSendMailAcknowledgementView
    LogoffView
    OptionsView
);

my @VIEW_DISPLAY_PARAMS = (
    -INPUT_WIDGET_DEFINITIONS   => \@INPUT_WIDGET_DEFINITIONS,
    -COLOR_FOR_EVEN_ROWS     => 'E5E5E5',
    -COLOR_FOR_ODD_ROWS      => 'FFFFFF',
    -APPLICATION_LOGO        => 'logo.gif',
    -APPLICATION_LOGO_HEIGHT => '40',
    -APPLICATION_LOGO_WIDTH  => '353',
    -APPLICATION_LOGO_ALT    => 'WebDB Demo',
    -HTTP_HEADER_PARAMS      => [-EXPIRES => '-1d'],
    -DOCUMENT_ROOT_URL       => '/',
    -IMAGE_ROOT_URL          => '/Images/Extropia',
    -LINK_TARGET             => '_self',
    -SCRIPT_DISPLAY_NAME     => 'MLM',
    -SCRIPT_NAME             => $CGI->script_name(),
    -EMAIL_DISPLAY_FIELDS    => \@EMAIL_DISPLAY_FIELDS,
    -HOME_VIEW               => 'MLMTOCView',
    -FIELD_NAME_MAPPINGS     => {
        fname      => 'First Name',
        lname      => 'Last Name',
        email      => 'Email',
        gender     => 'Gender',
        age        => 'Age',
        occupation => 'Occupation'
        },
    -DISPLAY_FIELDS        => [qw(
        fname
        lname
        email
        gender
        age
        occupation
        )],
    -SELECTED_DISPLAY_FIELDS        => [qw(
        fname
        lname
        email
        gender
        age
        occupation
        )],
    -SORT_FIELDS             => [qw(
        fname
        lname
        email
        gender
        age
        occupation
        )]
);  

######################################################################
#                           FILTER SETUP                             #
######################################################################

my @HTMLIZE_FILTER_CONFIG_PARAMS = (
    -TYPE            => 'HTMLize',
    -CONVERT_DOUBLE_LINEBREAK_TO_P => 1,
    -CONVERT_LINEBREAK_TO_BR => 1,
);

my @CHARSET_FILTER_CONFIG_PARAMS = (
    -TYPE            => 'CharSet'
);

my @EMBED_FILTER_CONFIG_PARAMS = (
    -TYPE            => 'Embed',
    -ENABLE          => 0
);

my @VIEW_FILTERS_CONFIG_PARAMS = (
     \@HTMLIZE_FILTER_CONFIG_PARAMS,
     \@CHARSET_FILTER_CONFIG_PARAMS,
     \@EMBED_FILTER_CONFIG_PARAMS
);


######################################################################
#                      ACTION/WORKFLOW SETUP                         #
######################################################################

my @ACTION_HANDLER_LIST = qw(
    DisplayCSSViewAction
    DisplaySessionTimeoutErrorAction
    DownloadFileAction
    PerformLogoffAction
    PerformLogonAction
    DisplayOptionsFormAction
    MLM/DisplayRecordAction
    MLM/DisplaySearchForRecordsFormAction
    DisplayAddFormAction
    DisplayAddRecordConfirmationAction
    ProcessAddRequestAction
    DisplayDeleteFormAction
    DisplayDeleteRecordConfirmationAction
    ProcessDeleteRequestAction
    DisplayModifyFormAction
    ProcessModifyRequestAction
    DisplayModifyRecordConfirmationAction
    MLM/DisplayMailFormAction
    MLM/ProcessMailSendAction
    DisplayPowerSearchFormAction
    DisplayDetailsViewAction
    PerformPowerSearchAction
    DisplayBasicDataViewAction
    DefaultAction
);

my @ACTION_HANDLER_ACTION_PARAMS = (
    -ACTION_HANDLER_LIST                    => \@ACTION_HANDLER_LIST,
    -FIELDS_REQUIRED_FOR_DELETION           => \@FIELDS_REQUIRED_FOR_DELETION,
    -ADD_ACKNOWLEDGEMENT_VIEW_NAME          => 'AddAcknowledgementView',
    -DELETE_ACKNOWLEDGEMENT_VIEW_NAME       => 'DeleteAcknowledgementView',
    -MODIFY_ACKNOWLEDGEMENT_VIEW_NAME       => 'ModifyAcknowledgementView',
    -POWER_SEARCH_VIEW_NAME                 => 'PowerSearchFormView',
    -ADD_RECORD_CONFIRMATION_VIEW_NAME      => 'AddRecordConfirmationView',
    -MODIFY_RECORD_CONFIRMATION_VIEW_NAME   => 'ModifyRecordConfirmationView',
    -DELETE_RECORD_CONFIRMATION_VIEW_NAME   => 'DeleteRecordConfirmationView',
    -ALLOW_ADDITIONS_FLAG                   => 1,
    -ALLOW_DELETIONS_FLAG                   => 1,
    -ALLOW_MODIFICATIONS_FLAG               => 1,
    -ALLOW_LIST_MAILINGS_FLAG               => 0,
    -ALLOW_DUPLICATE_ENTRIES                => 0,
    -AUTH_MANAGER_CONFIG_PARAMS             => \@AUTH_MANAGER_CONFIG_PARAMS,
    -ADD_FORM_VIEW_NAME                     => 'AddRecordView',
    -ADD_EMAIL_BODY_VIEW                    => 'AddEventEmailView',
    -OPTIONS_FORM_VIEW_NAME                 => 'OptionsView',
    -BASIC_DATA_VIEW_NAME                   => 'MLMTOCView',
    -CGI_OBJECT                             =>  $CGI,
    -CSS_VIEW_URL   => $CGI->script_name() . "?display_css_view=on",
    -CSS_VIEW_NAME  => "CSSView",
    -DATA_HANDLER_MANAGER_CONFIG_PARAMS     => \@DATA_HANDLER_MANAGER_CONFIG_PARAMS,
    -DATASOURCE_CONFIG_PARAMS               => \@DATASOURCE_CONFIG_PARAMS,
    -DETAILS_VIEW_NAME                      => 'DetailsView',
    -DELETE_FORM_VIEW_NAME                  => 'MLMFindRecordToDeleteFormView',
    -DELETE_EMAIL_BODY_VIEW                 => 'DeleteEventEmailView',
    -DEFAULT_SORT_FIELD1                    => 'date_time_posted',
    -DEFAULT_SORT_FIELD2                    => 'subject',
    -DEFAULT_VIEW_NAME                      => 'MLMTOCView',
    -DISPLAY_ACKNOWLEDGEMENT_ON_ADD_FLAG    => 1,
    -DISPLAY_ACKNOWLEDGEMENT_ON_DELETE_FLAG => 1,
    -DISPLAY_ACKNOWLEDGEMENT_ON_MODIFY_FLAG => 1,
    -DISPLAY_CONFIRMATION_ON_ADD_FLAG       => 1,
    -DISPLAY_CONFIRMATION_ON_DELETE_FLAG    => 0,
    -DISPLAY_CONFIRMATION_ON_MODIFY_FLAG    => 1,
    -ENABLE_SORTING_FLAG                    => 0,
    -HIDDEN_ADMIN_FIELDS_VIEW_NAME          => 'HiddenAdminFieldsView',
    -URL_ENCODED_ADMIN_FIELDS_VIEW_NAME     => 'URLEncodedAdminFieldsView',
    -LOG_CONFIG_PARAMS                      => \@LOG_CONFIG_PARAMS,
    -LOGOFF_VIEW_NAME                       => 'LogoffView',
    -MODIFY_FORM_VIEW_NAME                  => 'ModifyRecordView',
    -MODIFY_EMAIL_BODY_VIEW                 => 'ModifyEventEmailView',
    -MAIL_CONFIG_PARAMS                     => \@MAIL_CONFIG_PARAMS,
    -MAIL_SEND_PARAMS                       => \@MAIL_SEND_PARAMS,
    -REQUIRE_AUTH_FOR_SEARCHING_FLAG        => 0,
    -REQUIRE_AUTH_FOR_ADDING_FLAG           => 0,
    -REQUIRE_AUTH_FOR_MODIFYING_FLAG        => 0,
    -REQUIRE_AUTH_FOR_DELETING_FLAG         => 0,
    -REQUIRE_AUTH_FOR_VIEWING_DETAILS_FLAG  => 0,
    -REQUIRE_AUTH_FOR_SENDING_MAIL_FLAG     => 1,
    -REQUIRE_MATCHING_USERNAME_FOR_MODIFICATIONS_FLAG => 0,
    -REQUIRE_MATCHING_USERNAME_FOR_DELETIONS_FLAG     => 0,
    -REQUIRE_MATCHING_GROUP_FOR_MODIFICATIONS_FLAG    => 0,
    -REQUIRE_MATCHING_GROUP_FOR_DELETIONS_FLAG        => 0,
    -REQUIRE_MATCHING_USERNAME_FOR_SEARCHING_FLAG     => 0,
    -REQUIRE_MATCHING_GROUP_FOR_SEARCHING_FLAG        => 0,
    -SEND_EMAIL_ON_DELETE_FLAG              => 0,
    -SEND_EMAIL_ON_MODIFY_FLAG              => 0,
    -SEND_EMAIL_ON_ADD_FLAG                 => 0,
    -SEND_MAIL_ACKNOWLEDGEMENT_VIEW_NAME    => 'MLMSendMailAcknowledgementView',
    -SEND_MAIL_FORM_VIEW_NAME               => 'MLMSendMailFormView',
    -SESSION_OBJECT                         => $SESSION,
    -SESSION_TIMEOUT_VIEW_NAME              => 'SessionTimeoutErrorView',
    -SIMPLE_SEARCH_BOX_VIEW_NAME            => 'SimpleSearchBoxView',
    -SORT_DIRECTION                         => 'DESC',
    -USE_XML                                => $CGI->param('use_xml') || 0,
    -USE_WAP                                => $CGI->param('use_wap') || 0,
    -VIEW_FILTERS_CONFIG_PARAMS             => \@VIEW_FILTERS_CONFIG_PARAMS,
    -VIEW_DISPLAY_PARAMS                    => \@VIEW_DISPLAY_PARAMS,
    -VALID_VIEWS                            => \@VALID_VIEWS,
    -VIEW_LOADER                            => $VIEW_LOADER,
    -MAX_RECORDS_PER_PAGE                   => $CGI->param('records_per_page') || 10,
    -SORT_FIELD1                            => $CGI->param('sort_field1') || 'category',
    -SORT_FIELD2                            => $CGI->param('sort_field2') || 'fname',
    -SIMPLE_SEARCH_STRING => $CGI->param('simple_search_string') || "",
    -FIRST_RECORD_ON_PAGE => $CGI->param('first_record_to_display') || 0,
    -LAST_RECORD_ON_PAGE  => $CGI->param('first_record_to_display') || "0",
    -KEY_FIELD               => 'record_id',
    -PAGE_TOP_VIEW           => 'PageTopView',
    -PAGE_BOTTOM_VIEW           => 'PageBottomView'
);

######################################################################
#                      LOAD APPLICATION                              #
######################################################################

my $APP = new Extropia::App::DBApp(
    -ACTION_HANDLER_ACTION_PARAMS => \@ACTION_HANDLER_ACTION_PARAMS,
    -ACTION_HANDLER_LIST          => \@ACTION_HANDLER_LIST,
    -VIEW_DISPLAY_PARAMS          => \@VIEW_DISPLAY_PARAMS
    ) or die("Unable to construct the application object in " . 
             $CGI->script_name() . " Please contact the webmaster.");

print $APP->execute();
